@@ -151,6 +151,7 @@ DEFAULT_CONFIG_DICT = {
     "benchmark": False,  # When True, a simple benchmark will be run to estimate useful timing metrics
     "benchmark_polyak": 0.1,  # Polyak averaging factor for the benchmarks (0.0 < x <= 1); smaller is slower, bigger is noisier
     "wait_on_done": False,  # Whether the wait() method should be called when either terminated or truncated is True
+    "last_act_on_reset": False,  # When False, reset() sends the default action; when False, it sends the last action
 }
 """Default configuration dictionary of Real-Time Gym.
 
@@ -237,7 +238,7 @@ class Benchmark:
         """before join().
         """
         self.__b_lock.acquire()
-        now = time.time()
+        now = time.perf_counter()
         if self.__end_step_time is not None:
             self.inference_duration, self.inference_duration_dev = self.running_average(new_val=now - self.__end_step_time, old_avg=self.inference_duration, old_avg_dev=self.inference_duration_dev)
         self.__start_step_time = now
@@ -247,7 +248,7 @@ class Benchmark:
         """before return.
         """
         self.__b_lock.acquire()
-        now = time.time()
+        now = time.perf_counter()
         if self.__start_step_time is not None:
             self.step_duration, self.step_duration_dev = self.running_average(new_val=now - self.__start_step_time, old_avg=self.step_duration, old_avg_dev=self.step_duration_dev)
         self.__end_step_time = now
@@ -257,7 +258,7 @@ class Benchmark:
         """before run_time_step.
         """
         self.__b_lock.acquire()
-        now = time.time()
+        now = time.perf_counter()
         if self.__start_time_step_time is not None:
             self.time_step_duration, self.time_step_duration_dev = self.running_average(new_val=now - self.__start_time_step_time, old_avg=self.time_step_duration, old_avg_dev=self.time_step_duration_dev)
         if self.__start_step_time is not None:
@@ -267,13 +268,13 @@ class Benchmark:
 
     def start_retrieve_obs_time(self):
         self.__b_lock.acquire()
-        now = time.time()
+        now = time.perf_counter()
         self.__start_retrieve_obs_time = now
         self.__b_lock.release()
 
     def end_retrieve_obs_time(self):
         self.__b_lock.acquire()
-        now = time.time()
+        now = time.perf_counter()
         if self.__start_retrieve_obs_time is not None:
             self.retrieve_obs_duration, self.retrieve_obs_duration_dev = self.running_average(new_val=now - self.__start_retrieve_obs_time, old_avg=self.retrieve_obs_duration, old_avg_dev=self.retrieve_obs_duration_dev)
         self.__end_retrieve_obs_time = now
@@ -281,7 +282,7 @@ class Benchmark:
 
     def end_send_control_time(self):
         self.__b_lock.acquire()
-        now = time.time()
+        now = time.perf_counter()
         if self.__start_time_step_time is not None:
             self.send_control_duration, self.send_control_duration_dev = self.running_average(new_val=now - self.__start_time_step_time, old_avg=self.send_control_duration, old_avg_dev=self.send_control_duration_dev)
         self.__end_send_control_time = now
@@ -304,6 +305,7 @@ class RealTimeEnv(Env):
 
         # config variables:
         self.wait_on_done = config["wait_on_done"] if "wait_on_done" in config else False
+        self.last_act_on_reset = config["last_act_on_reset"] if "last_act_on_reset" in config else False
         self.act_prepro_func: callable = config["act_prepro_func"] if "act_prepro_func" in config else None
         self.obs_prepro_func = config["obs_prepro_func"] if "obs_prepro_func" in config else None
         self.ep_max_length = config["ep_max_length"]
@@ -328,7 +330,6 @@ class RealTimeEnv(Env):
         self.__obs = None
         self.__rew = None
         self.__terminated = None
-        self.__truncated = None
         self.__info = None
         self.__o_set_flag = False
 
@@ -345,9 +346,10 @@ class RealTimeEnv(Env):
         self.observation_space = self._get_observation_space()
         self.current_step = 0
         self.time_initialized = False
+        self.running = False
+
         # state variables:
         self.default_action = self.interface.get_default_action()
-        self.last_action = self.default_action
 
         # gymnasium variables:
         self.seed = None
@@ -366,7 +368,7 @@ class RealTimeEnv(Env):
         It is recommended to draw a time diagram of your system
             action computation and observation capture can be performed in parallel
         """
-        now = time.time()
+        now = time.perf_counter()
         if now < self.__t_end + self.time_step_timeout:  # if either still in the previous time-step of within its allowed elasticity
             self.__t_start = self.__t_end  # the new time-step starts when the previous time-step is supposed to finish or to have finished
         else:  # if after the allowed elasticity
@@ -401,7 +403,7 @@ class RealTimeEnv(Env):
 
     def _initialize_time(self):
         """This is called at first reset()."""
-        now = time.time()
+        now = time.perf_counter()
         # fake a "previous" time step:
         self.__t_start = now - self.time_step_duration
         self.__t_co = self.__t_start + self.start_obs_capture
@@ -429,7 +431,7 @@ class RealTimeEnv(Env):
         if self.benchmark:
             self.bench.end_send_control_time()
         self._update_timestamps()
-        now = time.time()
+        now = time.perf_counter()
         if now < self.__t_co:  # wait until it is time to capture observation
             time.sleep(self.__t_co - now)
         if self.benchmark:
@@ -437,7 +439,7 @@ class RealTimeEnv(Env):
         self.__update_obs_rew_terminated_truncated()  # capture observation
         if self.benchmark:
             self.bench.end_retrieve_obs_time()
-        now = time.time()
+        now = time.perf_counter()
         if now < self.__t_end:  # wait until the end of the time-step
             time.sleep(self.__t_end - now)
 
@@ -449,27 +451,26 @@ class RealTimeEnv(Env):
         """
         self.__o_lock.acquire()
         o, r, d, i = self.interface.get_obs_rew_terminated_info()
-        t = (self.current_step >= self.ep_max_length) if not d else False
         elt = o
         if self.obs_prepro_func:
             elt = self.obs_prepro_func(elt)
         elt = tuple(elt)
-        self.__obs, self.__rew, self.__terminated, self.__truncated, self.__info = elt, r, d, t, i
+        self.__obs, self.__rew, self.__terminated, self.__info = elt, r, d, i
         self.__o_set_flag = True
         self.__o_lock.release()
 
-    def _retrieve_obs_rew_terminated_truncated_info(self):
+    def _retrieve_obs_rew_terminated_info(self):
         """Waits for new available o r d i and retrieves them.
         """
         c = True
         while c:
             self.__o_lock.acquire()
             if self.__o_set_flag:
-                elt, r, d, t, i = self.__obs, self.__rew, self.__terminated, self.__truncated, self.__info
+                elt, r, d, i = self.__obs, self.__rew, self.__terminated, self.__info
                 self.__o_set_flag = False
                 c = False
             self.__o_lock.release()
-        return elt, r, d, t, i
+        return elt, r, d, i
 
     def init_action_buffer(self):
         for _ in range(self.act_buf_len):
@@ -500,6 +501,7 @@ class RealTimeEnv(Env):
             info: info dictionary
         """
         self._join_thread()
+        self.running = True
         self.seed = seed
         self.options = options
         self.current_step = 0
@@ -508,7 +510,8 @@ class RealTimeEnv(Env):
             self.init_action_buffer()
         else:
             # replace the last (non-applied) action from the previous episode by the action that is going to be applied:
-            self.act_buf[-1] = self.default_action
+            if not self.last_act_on_reset:
+                self.act_buf[-1] = self.default_action
         elt, info = self.interface.reset(seed=seed, options=options)
         if self.act_in_obs:
             elt = elt + list(self.act_buf)
@@ -544,12 +547,18 @@ class RealTimeEnv(Env):
         self.act_buf.append(action)  # the action is always appended to the buffer
         if not self.real_time:
             self._run_time_step(action)
-        obs, rew, terminated, truncated, info = self._retrieve_obs_rew_terminated_truncated_info()
+        if self.running:
+            obs, rew, terminated, info = self._retrieve_obs_rew_terminated_info()
+        else:
+            raise RuntimeError("The episode is terminated or truncated. Call reset before step.")
+        truncated = (self.current_step >= self.ep_max_length) if not terminated else False
         done = (terminated or truncated)
         if not done:  # apply action only when not done
             self._run_time_step(action)
-        elif self.wait_on_done:
-            self.wait()
+        else:
+            self.running = False
+            if self.wait_on_done:
+                self.wait()
         if self.act_in_obs:
             obs = tuple((*obs, *tuple(self.act_buf),))
         if self.benchmark:
@@ -583,9 +592,25 @@ class RealTimeEnv(Env):
         Args:
             mode: not used
             join_thread: set this to True if your render method performs unsafe operations.
-                The render method of your interface is called outside of the Real-Time Gym thread.
+                The render method of your interface is called outside the Real-Time Gym thread.
                 Caution: when join_thread is True, render() is not compatible with benchmarks().
         """
         if join_thread:
             self._join_thread()
         self.interface.render()
+
+    def set_default_action(self, default_action):
+        """Changes the default action.
+
+        Use this method right before calling reset() if you want the environment to send another default_action.
+        This is useful when you want to maintain the real-time flow around the end of an episode.
+        For instance, you may want to call set_default_action() with default_action as the action sent to step() right
+        before the episode got terminated or truncated, because this action was never applied (thus, it will be applied
+        by reset() - note however that this last action can be random unless you take special care).
+
+        Note: alternatively, you can set the "last_act_on_reset" entry to True in the rtgym configuration.
+
+        Args:
+            default_action: numpy.array: new default action (make sure it complies with the action space)
+        """
+        self.default_action = default_action
